﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing;
using System.Drawing.Imaging;

namespace SequenceScreencapper
{
    public enum TargetFormat
    {
        Png,
        JPeg,
        Gif,
        Bmp
    }

    public class Program : Form
    {
        public static int display;
        public static int seconds = 4;
        public static string dir;
        public static TargetFormat format = TargetFormat.Png;

        public static Timer t;

        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            dir = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures);

            display = -1;
            
            Application.Run(new Program());

        }

        private NotifyIcon ico;
        private ContextMenuStrip menu;

        public Program()
        {
            menu = new ContextMenuStrip();
            menu.Items.Add("Start", Properties.Resources.Play.ToBitmap(), OnStart);
            menu.Items.Add("Config", null, OnConfig);
            menu.Items.Add("-");
            menu.Items.Add("Exit", SystemIcons.Error.ToBitmap(), OnExit);

            ico = new NotifyIcon();
            ico.Text = "Sequence Screencapper";
            ico.Icon = Properties.Resources.Play;

            ico.ContextMenuStrip = menu;
            ico.Visible = true;

            config = new Form1();
            config.Show();
        }

        Form1 config;

        private void OnStop(object sender, EventArgs e)
        {
            menu.Items[0].Text = "Start";
            menu.Items[0].Click -= OnStop;
            menu.Items[0].Click += OnStart;

            if (t != null)
            {
                t.Stop();
                t.Dispose();
            }
        }

        private void OnStart(object sender, EventArgs e)
        {
            menu.Items[0].Text = "Stop";
            menu.Items[0].Click -= OnStart;
            menu.Items[0].Click += OnStop;

            if (t != null)
                t = null;

            t = new Timer();
            t.Interval = seconds * 1000;
            t.Enabled = true;
            t.Tick += delegate
            {
                Screen scrn = null;
                try
                {
                    if (display < 0)
                        scrn = Screen.PrimaryScreen;
                    else
                        scrn = Screen.AllScreens[display];
                }
                catch (Exception ee)
                {
                    OnStop(sender, e);
                    MessageBox.Show("Can't capture display! Exception: " + ee.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

                Rectangle rect = scrn.Bounds;

                if (display == -2)
                {
                    int minx = 0;
                    int miny = 0;
                    int maxx = 0;
                    int maxy = 0;

                    for (int i = 0; i < Screen.AllScreens.Count(); i++)
                    {
                        Screen s = Screen.AllScreens[i];

                        if (minx > s.Bounds.X)
                            minx = s.Bounds.X;
                        if (miny > s.Bounds.Y)
                            miny = s.Bounds.Y;
                        if (maxx < s.Bounds.X + s.Bounds.Width)
                            maxx = s.Bounds.X + s.Bounds.Width;
                        if (maxy < s.Bounds.Y + s.Bounds.Height)
                            maxy = s.Bounds.Y + s.Bounds.Height;
                    }

                    rect = new Rectangle(minx, miny, maxx - minx, maxy - miny);
                }

                try
                {
                    Bitmap bmp = new Bitmap(rect.Width, rect.Height, PixelFormat.Format32bppArgb);

                    Graphics g = Graphics.FromImage(bmp);
                    g.CopyFromScreen(rect.X, rect.Y, 0, 0, rect.Size, CopyPixelOperation.SourceCopy);

                    string target = dir + "\\" + DateTime.Now.ToString("yyyy-MM-dd_hh-mm-ss");

                    switch (Program.format)
                    {
                        case TargetFormat.Png:  bmp.Save(target + ".png", ImageFormat.Png); break;
                        case TargetFormat.JPeg: bmp.Save(target + ".jpg", ImageFormat.Jpeg); break;
                        case TargetFormat.Gif:  bmp.Save(target + ".gif", ImageFormat.Gif); break;
                        case TargetFormat.Bmp:  bmp.Save(target + ".bmp", ImageFormat.Bmp); break;
                    }

                    bmp.Dispose();
                    g.Dispose();
                    scrn = null;
                }
                catch (Exception ee)
                {
                    if (ee.Message == "The handle is invalid")
                        return;

                    OnStop(sender, e);
                    MessageBox.Show("Unable to save image to disk. Exception: " + ee.Message, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }

            };
        }

        private void OnConfig(object sender, EventArgs e)
        {
            config = new Form1();
            config.Show();
        }

        private void OnExit(object sender, EventArgs e)
        {
            Application.Exit();
        }

        protected override void OnLoad(EventArgs e)
        {
            Visible = false;
            ShowInTaskbar = false;
            
            base.OnLoad(e);
        }

        protected override void Dispose(bool disposing)
        {
            if (disposing)
                ico.Dispose();

            base.Dispose(disposing);
        }
    }
}
